package com.example.android.ar.simple;

import java.io.IOException;
import java.util.List;

import android.content.Context;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.Toast;

/**
 * カメラのプレビュー画像を表示するためのクラスです。
 * 
 */
public class CameraPreviewView extends SurfaceView implements SurfaceHolder.Callback {

    private Camera camera;

    public CameraPreviewView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init();
    }

    public CameraPreviewView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public CameraPreviewView(Context context) {
        super(context);
        init();
    }

    /**
     * 　SurfaceHolderを初期化します
     * 
     */
    protected void init() {
        SurfaceHolder surfaceHolder = getHolder();
        surfaceHolder.addCallback(this);
        surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        if (camera != null) {
            // Surfaceが変更されたら一旦プレビューを停止します
            camera.stopPreview();

            // 変更されたSurfaceにあわせてプレビューのサイズを変更します
            setPreviewSize(width, height);
            // プレビューを開始します
            camera.startPreview();
        }
    }

    public void surfaceCreated(SurfaceHolder holder) {
        if (camera == null) {
            try {
                // Surfaceが生成されるときに、カメラを取得します。
                camera = Camera.open();
            } catch (RuntimeException e) {
                Toast.makeText(getContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
                return;
            }
        }

        try {
            // カメラのプレビュー画像の表示先を設定します。
            camera.setPreviewDisplay(holder);
        } catch (IOException e) {
            // 設定に失敗したらカメラを解放します。
            camera.release();
            camera = null;
            Toast.makeText(getContext(), e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        if (camera != null) {
            // Surfaceが破棄されるときに、プレビューを停止してカメラを解放します。
            camera.stopPreview();
            camera.release();
            camera = null;
        }
    }

    /**
     * プレビュー画像のサイズを設定します。
     * 
     * @param width
     * @param height
     */
    void setPreviewSize(int width, int height) {
        Camera.Parameters params = camera.getParameters();
        // 端末のカメラでサポートされているプレビュー画像のサイズを取得します。
        List<Camera.Size> supported = params.getSupportedPreviewSizes();
        if (supported != null) {
            for (Camera.Size size : supported) {
                // サポートされているサイズの中から、画面のサイズと同じ（または小さい）ものを選択します。
                if (size.width <= width && size.height <= height) {
                    params.setPreviewSize(size.width, size.height);
                    camera.setParameters(params);
                    break;
                }
            }
        }
    }
}
