package com.example.android.ar.simple;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import android.content.Context;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.location.Location;
import android.location.LocationListener;
import android.net.Uri;
import android.os.Bundle;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

/**
 * カメラ画像に位置情報をオーバーレイ表示するためのクラスです。
 * 
 */
public class LocationsView extends View implements LocationListener, OrientationEventListener {

    /** 遠くにある位置情報オブジェクトを上方に表示するための値(距離 / DEPTH)分だけ画面の上方に表示します */
    private static final float DEPTH = 5;

    // nexus one
    // h:54.8
    // v:42.5

    // 35mm
    // h:64params
    // v:45

    /** 端末の描画範囲(°) **/
    private static final float ANGLE = 54.8f;

    /** 位置情報のラベルを表示するためのPaint */
    Paint paint = new Paint();

    /** 端末の座標 */
    private Location deviceLocation;

    /** 位置情報オブジェクトのリスト */
    private List<ProximityObject> proximityList = new CopyOnWriteArrayList<ProximityObject>();

    /** 　端末の方位角 */
    private float azimuth;
    /** 端末の傾斜 */
    private float pitch;
    /** 端末の回転 */
    private float roll;

    /** 端末の座標に対応する住所 */
    private String address;

    public LocationsView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init();
    }

    public LocationsView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public LocationsView(Context context) {
        super(context);
        init();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        synchronized (this) {

            // 現在位置の住所を表示します。
            if (address != null) {
                canvas.drawText(address, 20, 20, paint);
            }

            for (ProximityObject p : proximityList) {
                // 位置情報オブジェクトを表示します。
                drawProximity(canvas, p);
            }
        }
    }

    /**
     * 位置情報が持つ描画情報を更新します。
     * 
     */
    protected void updateProximities() {
        final Location location = deviceLocation;

        // 端末の座標が取得できていない場合は、位置情報の相対位置が計算できないので何もしません
        if (location == null)
            return;

        // 位置情報間(端末の位置と目標の位置)の距離を取得するためのバッファ
        float[] distance = new float[3];

        for (ProximityObject po : proximityList) {
            // 位置情報間の距離を計算します
            Location.distanceBetween(location.getLatitude(), location.getLongitude(), po.latitude, po.longitude, distance);
            // 位置情報に反映します。
            updateProximity(po, distance);
        }
    }

    /**
     * 文字列を表示するためのPaintを初期化します。
     * 
     * @param context
     * 
     */
    protected void init() {
        paint.setAntiAlias(true);
        paint.setTextSize(12);
        paint.setColor(Color.WHITE);
    }

    /**
     * 傾きの変化に応じて、位置情報を更新します。
     * 
     * @param proximity
     * @param distance
     */
    private void updateProximity(ProximityObject proximity, float[] distance) {
        // 画面の幅
        float width = getWidth();
        // 画面の高さ
        float height = getHeight();

        // 端末から位置情報に対する方向(0〜360[degrees])のradians。
        double pointAzimuth = Math.toRadians(distance[1]);

        // 端末が向いている方向(-180〜180[radians])
        // 計算を簡単にするために方向の基準をpointAzimath(目標の位置情報に対する方向)にあわせる
        double centerAzimuth = azimuth < 0 ? azimuth + Math.PI * 2 : azimuth;

        // 方向の差
        double theta;
        // コメント中の C はcenterAzimuth、B はpointAzimuth

        // C < B
        if (centerAzimuth < pointAzimuth) {

            if (centerAzimuth - pointAzimuth < Math.PI) {
                // C < B のとき
                // C - B < 180 ならば
                // B は C(A) から時計回りが近い
                // この場合の角度は B - C
                theta = pointAzimuth - centerAzimuth;
            } else {
                // C < B のとき
                // B - C > 180 ならば
                // B は C(A) から反時計回りが近い
                // この場合の角度は (B - C) - 360
                theta = (pointAzimuth - centerAzimuth) - 2 * Math.PI;
            }
        } else {
            // B > C

            if (centerAzimuth - pointAzimuth < Math.PI) {
                // C - B < 180 ならば
                // B は C(A) から半時計回りが近い
                // B - C
                theta = pointAzimuth - centerAzimuth;
            } else {
                // C - B > 180 ならば B は C(A) から時計回りが近い
                // この場合の角度は B - A + 360
                theta = (pointAzimuth - centerAzimuth) + 2 * Math.PI;
            }
        }

        // カメラの画角の外にある位置情報を描画しないようにします。
        if (Math.abs(theta) > ANGLE / 2) {
            proximity.display = false;
            return;
        }

        proximity.display = true;

        // 画面の中心のｘ座標
        int viewCenter = (int) (width / 2);

        // 位置情報を表示するx座標
        // thetaには地理情報オブジェクトが端末が向いている方角より時計回り方向にある場合は正の値、反時計回り方向にある場合は負の値が設定されています
        // この値は、地理情報がカメラの正面を0とした場合の角度の差です
        int x = (int) (viewCenter + (width / Math.toRadians(ANGLE)) * theta);

        // 画面の下限に余白を設定します。
        float borderY = (int) (getHeight() * 0.9);

        // 遠くにあるものを画面の上方に表示します
        int delta = (int) (distance[0] / DEPTH);

        // 　端末の傾きにあわせてY軸を補正します
        int trim = (int) (Math.sin(pitch) * height / 2);

        // 位置情報を表示するy座標
        int y = (int) (borderY - delta - trim);

        // 位置情報の描画領域を更新します
        proximity.rect.left = x - proximity.drawer.getWidth() / 2;
        proximity.rect.top = y - proximity.drawer.getHeight() / 2;
        proximity.rect.right = proximity.rect.left + proximity.drawer.getWidth();
        proximity.rect.bottom = proximity.rect.top + proximity.drawer.getHeight();

        // set(, y, x + proximity.drawer.getWidth(), y +
        // proximity.drawer.getHeight());
    }

    /**
     * 画面に位置情報を表示します。
     * 
     * @param canvas
     * @param proximity
     */
    private void drawProximity(Canvas canvas, ProximityObject proximity) {
        // 位置情報がカメラが向いている方向にない場合は、表示しません
        if (!proximity.display)
            return;

        // 位置情報をアイコンとして表示します。
        canvas.drawBitmap(proximity.drawer, proximity.rect.left, proximity.rect.top, null);

        // アイコンの下に位置情報の名前を表示します。
        int textX = proximity.rect.left;
        int textY = proximity.rect.bottom;
        canvas.drawText(proximity.name, textX, textY, paint);
    }

    /*
     * (non-Javadoc)
     * 
     * @see android.view.View#onTouchEvent(android.view.MotionEvent)
     */
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        synchronized (this) {

            // タッチの開始以外（画面に指が触れたとき）以外は無視します
            if (event.getAction() != MotionEvent.ACTION_DOWN)
                return super.onTouchEvent(event);

            for (ProximityObject p : proximityList) {
                // タップされた座標を取得します。
                int x = (int) event.getX();
                int y = (int) event.getY();

                // タップされた画面上の座標に位置情報が表示されている場合、位置情報のURLをブラウザで表示します。
                // 位置情報(Proximityクラス)は自身の描画領域をRectクラス型のメンバ変数rectとして保持しています。
                // Rectクラスのcontains(int,
                // int)メソッドを使えば、描画領域が(タップされた位置の)x,y座標を含んでいるかがわかります。
                if (p.rect.contains(x, y)) {
                    if (p.url == null)
                        continue;
                    // ACTION_VIEWにUriを指定したインテントを発行してブラウザを起動します。
                    Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(p.url));
                    getContext().startActivity(intent);
                    return true;
                }
            }

            return false;
        }
    }

    /**
     * 画面に表示する位置情報を更新します。
     * 
     * 更新された位置情報は、傾きが変化したときに描画されます。
     * 
     * @param result
     */
    public void updateProximityList(List<ProximityObject> result) {
        synchronized (this) {
            // 新しい位置情報を追加します。
            proximityList.addAll(result);
        }
    }

    /**
     * OrientationEventListenerインターフェイスのメソッドを実装して、端末の傾きの変換を取得します。
     * 
     * @see com.example.android.ar.simple.OrientationEventListener#onOrientationChanged(float,
     *      float, float)
     */
    public void onOrientationChanged(float azimuth, float pitch, float roll) {
        synchronized (this) {
            this.azimuth = azimuth;
            this.pitch = pitch;
            this.roll = roll;

            // 傾きの変化に応じて、画面を再描画します
            updateProximities();
            invalidate();
        }
    }

    public void onLocationChanged(Location location) {
        synchronized (this) {
            if (location == null)
                return;

            GeocoderTask geocoderTask = new GeocoderTask(getContext()) {
                @Override
                protected void onPostExecute(String result) {
                    synchronized (this) {
                        address = result;
                    }
                }
            };
            geocoderTask.execute(location.getLatitude(), location.getLongitude());

            // 端末の位置が変わった場合、位置情報をクリアして取得しなおします。
            proximityList.clear();
            deviceLocation = location;

            Context context = getContext();

            // 端末のまわりの八方位を地理情報として追加します
            new DirectionTask(context, this).execute(location);
        }
    }

    public void onProviderDisabled(String provider) {
    }

    public void onProviderEnabled(String provider) {
    }

    public void onStatusChanged(String provider, int status, Bundle extras) {
    }
}
