package com.example.android.ar.simple;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.WindowManager;

public class MyActivity extends Activity {

    private static final String TAG = "SimpleAR";

    /** 端末の座標の許容精度(m) */
    static final float ACCURACY_THRESHOLD = 100; // 100m
    /** 端末の座標を通知する間隔（秒） */
    static final int MIN_UPDATE_FLEQUENCY = 1 * 60 * 1000; // 1min
    /** 端末の座標を通知する最小距離（m） */
    static final int MIN_UPDATE_DISTANCE = 100; // 100m

    /** カメラ画像を表示するビューオブジェクト */
    SurfaceView cameraView;

    /** 位置情報をオーバーレイ表示するビューオブジェクト */
    LocationsView locationsView;

    /** センサーの値を表示するためのビューオブジェクト */
    OrientationsView orientationsView;

    /** 端末の傾きを捕捉するリスナーオブジェクト */
    MySensorEventListener orientationListener;

    /** センサーの登録有無 */
    boolean sensorRegistered;

    /** SensorManager。端末の傾きを取得するために使います。 */
    SensorManager sensorManager;

    /** LocationManager。端末の現在位置を取得するために使います。 */
    LocationManager locationManager;

    /** 高精度な端末の座標を短期間で捕捉するリスナーオブジェクト */
    LocationListener bounceLocationListener = new LocationListener() {

        public void onStatusChanged(String provider, int status, Bundle extras) {
        }

        public void onProviderEnabled(String provider) {
        }

        public void onProviderDisabled(String provider) {
        }

        public void onLocationChanged(Location location) {
            locationsView.onLocationChanged(location);
            if (location.getAccuracy() < ACCURACY_THRESHOLD) {
                // 許容される精度で座標が捕捉できたら、捕捉を終了します
                locationManager.removeUpdates(bounceLocationListener);
                locationManager.removeUpdates(coarseLocationListener);
            }
        }
    };

    /** 低精度な端末の座標を捕捉するリスナーオブジェクト */
    LocationListener coarseLocationListener = new LocationListener() {

        public void onStatusChanged(String provider, int status, Bundle extras) {
        }

        public void onProviderEnabled(String provider) {
        }

        public void onProviderDisabled(String provider) {
        }

        public void onLocationChanged(Location location) {
            locationsView.onLocationChanged(location);
            if (location.getAccuracy() < ACCURACY_THRESHOLD) {
                // 許容される精度で座標が捕捉できたら、捕捉を終了します
                locationManager.removeUpdates(coarseLocationListener);
            }
        }
    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        //画面をONに保ちます
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.main);

        // サービスを取得します
        sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

        // カメラ表示用のビューを初期化します
        cameraView = (SurfaceView) findViewById(R.id.camera);
        SurfaceHolder surfaceHolder = cameraView.getHolder();
        surfaceHolder.addCallback(new MySurfaceHolderCallback());
        surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        // オーバーレイ表示用のビューを取得します
        locationsView = (LocationsView) findViewById(R.id.locations);
        // センサー表示用のビューを取得します
        orientationsView = (OrientationsView) findViewById(R.id.orientations);
        // 傾きを検知するためのSensorEventListenerオブジェクトを作成します
        orientationListener = new MySensorEventListener(orientationsView, locationsView);
    }

    @Override
    protected void onResume() {
        super.onResume();

        // アプリケーションが前面になった場合にセンサーの捕捉を開始します。
        Sensor sensor;

        sensor = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
        if (sensor != null)
            sensorRegistered |= sensorManager.registerListener(orientationListener, sensor, SensorManager.SENSOR_DELAY_UI);

        sensor = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        if (sensor != null)
            sensorRegistered |= sensorManager.registerListener(orientationListener, sensor, SensorManager.SENSOR_DELAY_UI);

        // アプリケーションが前面になった場合に端末の座標の捕捉を開始します。

        // 高精度な位置情報を通常の間隔で捕捉するには時間がかかるので、
        // (1)高精度な位置情報を短期間で捕捉するリスナー
        // (2)低精度な位置情報を捕捉するリスナー
        // の二つを追加で登録し、短い時間で位置情報が取れるようにします。
        // 許容される精度の位置情報が取れた際には(1)、(2)の登録は解除します。
        Criteria criteria = new Criteria();
        criteria.setPowerRequirement(Criteria.POWER_LOW);

        // 高精度な位置情報を捕捉するためのリスナーを要求します。
        criteria.setAccuracy(Criteria.ACCURACY_FINE);
        String bestProvider = locationManager.getBestProvider(criteria, true);

        // 低精度な位置情報を捕捉するためのリスナーを要求します。
        criteria.setAccuracy(Criteria.ACCURACY_COARSE);
        String coarseProvider = locationManager.getBestProvider(criteria, true);

        locationManager.requestLocationUpdates(bestProvider, MIN_UPDATE_FLEQUENCY, MIN_UPDATE_DISTANCE, locationsView);
        locationManager.requestLocationUpdates(coarseProvider, 0, 0, coarseLocationListener);
        locationManager.requestLocationUpdates(bestProvider, 0, 0, bounceLocationListener);

        Log.d(TAG, "bestProvider=" + bestProvider);
        Log.d(TAG, "coarseProvider=" + coarseProvider);
    }

    @Override
    protected void onPause() {
        super.onPause();

        // アプリケーションが前面でなくなった場合はセンサーの捕捉を中断して、電池の消費を抑えます。
        if (sensorRegistered)
            sensorManager.unregisterListener(orientationListener);
        sensorRegistered = false;

        // アプリケーションが前面でなくなった場合は端末の座標の捕捉を中断して、電池の消費を抑えます。
        locationManager.removeUpdates(locationsView);
        locationManager.removeUpdates(coarseLocationListener);
        locationManager.removeUpdates(bounceLocationListener);
    }

    @Override
    public boolean onCreateOptionsMenu(android.view.Menu menu) {
        super.onCreateOptionsMenu(menu);
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.menu_insert_location:
            insertLocation();
            break;
        case R.id.menu_delete_location:
            deleteLocation();
            break;
        default:
            break;
        }

        return true;
    }

    private void deleteLocation() {
        // Applicationが処理中に応答しなくなるのを防ぐ
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected Void doInBackground(Void... params) {
                // データの削除をバックグラウンドで実行する
                // UIスレッドで行うとアプリケーションの応答性が低下する
                new LocationDatabase(MyActivity.this).deleteAll();
                return null;
            }

            @Override
            protected void onPostExecute(Void result) {
                Location location = ensureLastKnownLocation();
                locationsView.onLocationChanged(location);
            };
        }.execute();
    }

    /**
     * 最新の位置情報を取得します
     * 
     * @return
     */
    private Location ensureLastKnownLocation() {
        Location candidateLocation = null;
        for (String provider : locationManager.getAllProviders()) {
            Location lastKnownLocation = locationManager.getLastKnownLocation(provider);
            if (lastKnownLocation == null)
                continue;
            if (candidateLocation == null)
                candidateLocation = lastKnownLocation;
            if (candidateLocation.getTime() < lastKnownLocation.getTime())
                candidateLocation = lastKnownLocation;
        }
        return candidateLocation;
    }

    private void insertLocation() {
        final Timer timer = new Timer();
        final ProgressDialog progress = new ProgressDialog(this);
        final Location location = ensureLastKnownLocation();

        // 座標から住所をジオコーディングするための非同期タスクを実行します。
        final GeocoderTask geocoderTask = new GeocoderTask(this) {
            @Override
            protected void onPostExecute(String result) {
                // タイマーをキャンセルします。
                timer.cancel();
                // プログレスダイアログをキャンセルします。
                progress.cancel();

                new LocationDatabase(MyActivity.this).insert(location, result);
                locationsView.onLocationChanged(location);
            }
        };
        geocoderTask.execute(location.getLatitude(), location.getLongitude());

        // 一定時間(200ミリ秒)でジオコーディングが完了しなかった場合に、プログレスダイアログを表示します。
        TimerTask progressTask = new TimerTask() {
            @Override
            public void run() {
                progress.setMessage(getString(R.string.progress_message));
                progress.setCancelable(true);

                // プログレスダイアログがキャンセルされたらジオコーディングを中止します。
                DialogInterface.OnDismissListener listener;
                listener = new DialogInterface.OnDismissListener() {
                    public void onDismiss(DialogInterface dialog) {
                        geocoderTask.cancel(false);
                    }
                };
                progress.setOnDismissListener(listener);

                // ダイアログの表示処理をUIスレッドで実行します。
                runOnUiThread(new Runnable() {
                    public void run() {
                        progress.show();
                    }
                });
            }
        };
        timer.schedule(progressTask, 200);
    }
}